<?php
/**
* @copyright    Copyright (C) 2009 - 2015 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license      GNU/GPL, see LICENSE.php
* @package      PayPlans
* @subpackage   Frontend
* @contact      support+payplans@readybytes.in
*/

if(defined( '_JEXEC' )==false) {
	die( 'Restricted access' );
}

define("stateCode", 5);
class PayplansAppRobokassa extends PayplansAppPayment
{
	
	//inherited properties
	protected $_location	= __FILE__;

	function isApplicable($refObject = null, $eventName='')
	{
		// return true for event onPayplansControllerCreation
		if($eventName == 'onPayplansControllerCreation'){
			return true;
		}
		
		return parent::isApplicable($refObject, $eventName);
	}
	
	function onPayplansControllerCreation(&$view, &$controller, &$task, &$format)
	{
		if($view == 'payment' && ($task == 'notify' || $task == 'complete') ){
			$paymentKey = JFactory::getApplication()->input->get('Shp_paymentKey', null);
			if(!empty($paymentKey)){
				JFactory::getApplication()->input->set('payment_key', $paymentKey, 'POST');
				return true;
			}
		}
		return true;
	
	}
	
	function onPayplansPaymentForm(PayplansPayment $payment, $data = null)
	{		
		$invoice 				= $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$amount  				= $invoice->getTotal();
		$plans					= $invoice->getPlans(PAYPLANS_INSTANCE_REQUIRE);
		$plan					= array_pop($plans);
		$merchantLogin 			= $this->getAppParam('merchantLogin','');
		$merchantPassword 		= $this->getAppParam('merchantPassword1','');
		$paymentKey 			= $payment->getKey();

		$postUrl				= 'https://auth.robokassa.ru/Merchant/Index.aspx';
		if($this->getAppParam('SandBox')){
			$postUrl            ='http://test.robokassa.ru/Index.aspx'; 
		}
		$outSum					= number_format($invoice->getTotal(), 2);

		$invId					= $this->_getUniqInvId();
		$signature				= md5("$merchantLogin:$outSum:$invId:$merchantPassword:Shp_paymentKey=$paymentKey");		
		
		$this->assign('post_url', 			$postUrl);
		$this->assign('merchantLogin', 		$merchantLogin);
		$this->assign('invId', 				$invId);
		$this->assign('desc', 				$plan->getTitle());
		$this->assign('signature', 			$signature);
		$this->assign('outSum', 			$outSum);
		$this->assign('currencyLabel', 		$invoice->getCurrency('isocode'));
		$this->assign('Shp_paymentKey', 	$paymentKey);
		
		return $this->_render('form');
	}
	
	public function onPayplansPaymentNotify(PayplansPayment $payment, $data, $controller)
	{
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		
		//check for correct md5 of CRC
		if ($this->_validateCRC($data)==false) {
    		$message = XiText::_('COM_PAYPLANS_APP_ROBOKASSA_INVALID_CRC');
    		PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $data, 'PayplansPaymentFormatter', '', true);
    		return "COM_PAYPLANS_APP_ROBOKASSA_INVALID_CRC";
		} else {
				$txn_id 		= isset($data['InvId']) ? $data['InvId'] : 0;
	    		$subscr_id  	= 0;
	    		$parent_txn 	= 0;
	    	
	    	
			// get the transaction instace of lib
			$transaction = PayplansTransaction::getInstance();
			$transaction->set('user_id', 			$payment->getBuyer())
						->set('invoice_id', 		$invoice->getId())
						->set('payment_id', 		$payment->getId())
						->set('gateway_txn_id', 	$txn_id)
						->set('gateway_subscr_id', 	$subscr_id)
						->set('gateway_parent_txn', $parent_txn)
						->set('amount', 			0)
						->set('message', 			'COM_PAYPLANS_APP_ROBOKASSA_TRANSACTION_INITIATED')
						->set('params', 			PayplansHelperParam::arrayToIni($data));
					
			
		//store the response in the payment AND save the payment
		if(!$transaction->save()){
			$message = XiText::_('COM_PAYPLANS_LOGGER_ERROR_TRANSACTION_SAVE_FAILD');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $data, 'PayplansPaymentFormatter', '', true);
		}
		
		// if same notification came more than one time
	    	// Check if transaction already exists,if yes then do nothing and return
	    	
	    	//Special Case :- Since robokassa does not allow multiple notification for single transaction(invId) hence we are not handling
	    	//multiple transaction check here.

	    	//check the status of payment notification
			$response 		= $this->_getPaymentStatus($txn_id);
			
	    	$transactions 	= $this->_getExistingTransaction($invoice->getId(), $txn_id, $subscr_id, $parent_txn);
	    	if($transactions){
	    		foreach($transactions as $transaction){
	    				$transaction = PayplansTransaction::getInstance($transaction->transaction_id, null, $transaction);
	    				if($transaction->getParam('State_Code','') == $response['State_Code']){
	    					return true;
	    				}
		    		}
	    	}
		    	
		    	// get the transaction instace of lib
			$trans = PayplansTransaction::getInstance();
			$trans->set('user_id', 			$payment->getBuyer())
						->set('invoice_id', 		$invoice->getId())
						->set('payment_id', 		$payment->getId())
						->set('gateway_txn_id', 	$txn_id)
						->set('gateway_subscr_id', 	$subscr_id)
						->set('gateway_parent_txn', $parent_txn)
						->set('message', 			'COM_PAYPLANS_APP_ROBOKASSA_TRANSACTION_IN_PROCESS')
						->set('params', 			PayplansHelperParam::arrayToIni($data));
						
						
		if($response['State_Code'] == 100)
		{
			$trans->set('amount', 			$data['OutSum'])
				  ->set('message', 			'COM_PAYPLANS_APP_ROBOKASSA_TRANSACTION_COMPLETED');
		}
					
			
		//store the response in the payment AND save the payment
		if(!$trans->save()){
			$message = XiText::_('COM_PAYPLANS_LOGGER_ERROR_TRANSACTION_SAVE_FAILD');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $response, 'PayplansPaymentFormatter', '', true);
		}
		
		// Special case as in robokassa we have to return the 'OKInvId' from notify script. Where InvId is the unique invoice id. 		
		return 'OK'.$txn_id;
		}
	}
	
	
	/**
	 * This function will check the md5 of response and the one created by pass2.
	 * @param array $data
	 * @return boolean
	 */
	function _validateCRC(array $data )
    {
   		$pass2 		= $this->getAppParam('merchantPassword2','');
   		$crc 		= strtoupper($data['SignatureValue']);
   		$outSum 	= $data['OutSum'];
   		$InvId 		= $data['InvId'];
   		$paymentKey = $data['Shp_paymentKey'];
   		$my_crc 	= strtoupper(md5("$outSum:$InvId:$pass2:Shp_paymentKey=$paymentKey"));
   		if ($my_crc !=$crc) {
   			return false;
   		}
		return true;
    }
    
    
	public function onPayplansPaymentAfter(PayplansPayment $payment, &$action, &$data, $controller)
	{
		$logs   = PayplansHelperLogger::getLog($payment, XiLogger::LEVEL_ERROR);
		$record = array_pop($logs);				
		if($action =='error'){
			$invoice 		= $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
			$invoiceId 		= $data['InvId'];
			//check the status of payment notification
			$response 		= $this->_getPaymentStatus($invoiceId); 

		 	$txn_id 		= isset($data['InvId']) ? $data['InvId'] : 0;
	    	$subscr_id  	= 0;
	    	$parent_txn 	= 0;
	    	
			// if same notification came more than one time
		    	// Check if transaction already exists,if yes then do nothing and return
		    	$transactions 	= $this->_getExistingTransaction($invoice->getId(), $txn_id, $subscr_id, $parent_txn);
		    	if($transactions){
		    		foreach($transactions as $transaction){
	    				$transaction = PayplansTransaction::getInstance($transaction->transaction_id, null, $transaction);
	    				if($transaction->getParam('State_Code','') == $response['State_Code']){
	    					return true;
	    				}
		    		}
		    	}

	    		$transaction 	= PayplansTransaction::getInstance();
				$transaction->set('user_id', 	$payment->getBuyer())
					->set('invoice_id', 		$invoice->getId())
					->set('payment_id', 		$payment->getId())
					->set('gateway_txn_id', 	$txn_id)
					->set('gateway_subscr_id', 	$subscr_id)
					->set('gateway_parent_txn', $parent_txn)
					->set('amount', 			0)
					->set('message', 			'COM_PAYPLANS_APP_ROBOKASSA_TRANSACTION_NOT_COMPLETED')
					->set('params', 			PayplansHelperParam::arrayToIni($response));
					
			
			//store the response in the payment AND save the payment
			if(!$transaction->save()){
				$message 	= XiText::_('COM_PAYPLANS_LOGGER_ERROR_TRANSACTION_SAVE_FAILD');
				PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $response, 'PayplansPaymentFormatter', '', true);
			}
		}

		return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
	}


	/**
	 * This method converts the input xml to array.
	 *
	 * @param unknown_type $xml
	 * @return array
	 */
	function XML2Array ($xml)
	{

		$newArray = array () ;
		foreach ($xml as $key => $value) {
			if (is_array($value)){
				foreach($value as $key1 => $value1){
					if(is_array($value1)){
						foreach ($value1 as $key2 => $value2){
							$newArray[$key."_".$key2] = $value2;
						}
					}else {
							$newArray[$key."_".$key1] = $value1;	
					}
				}
					
			} else {
				$newArray[$key] = $value;
			}
		}
			
		return $newArray ;
	}
	
	
	 /**
	  * This will generate a new invId each time that is required by robokassa.
	  * @return uniqInvId
	  */
	protected function _getUniqInvId() 
	{
		$tempId 					= substr(time()* rand(),0, 7);
		$model 						= XiFactory::getInstance('transaction', 'model');
		$filter 					= array();
		$filter['gateway_txn_id']	= $tempId; 
		$result 					= $model->loadRecords($filter);
		
		if(count($result)){
			$tempId	 = $this->_getUniqInvId();
		} else {
			return $tempId;
		}
	}
	
	
	/**
	 * This function will return the payment status of each invoiceId.
	 * @param unknown_type $invoiceId
	 * @return Ambigous <multitype:, unknown>
	 */
	protected function _getPaymentStatus($invoiceId) 
	{
		$login 			= $this->getAppParam('merchantLogin','');
		$pass2 			= $this->getAppParam('merchantPassword2','');
		$signature 		= md5("$login:$invoiceId:$pass2");
		$url 			= 'https://auth.robokassa.ru/Merchant/WebService/Service.asmx/OpState?';
		$queryString 	= 'MerchantLogin='.$login.'&InvoiceID='.$invoiceId.'&Signature='.$signature;
			
		if($this->getAppParam('SandBox')){
				$url 			= 'http://test.robokassa.ru/Webservice/Service.asmx/OpState?';
				$queryString 	= 'MerchantLogin='.$login.'&InvoiceID='.$invoiceId.'&Signature='.$signature.'&StateCode='.stateCode;
		}
			
		$link 			= new JURI($url);	
		$curl 			= new JHttpTransportCurl(new JRegistry());
		$response 		= $curl->request('POST', $link,$queryString);
       	$content   		= $response->body;
		$xml 			= simplexml_load_string($content);
		$json 			= json_encode($xml);
		$response 		= json_decode($json,TRUE);
		$response 		= $this->XML2Array($response);
		
		return $response;
	}
}